# Common to both packages!
# Contains thermal functions and grid-based functions.

#' flips a 3D *array* about a given axis
#' @param img image array to flip
#' @param type the type of flip: flip horizontally (ie about vertical
#' axis), or vertically (about horizontal axis), or both.
#' @return the input array flipped about the specified dimension
#' @family reshape
imflip.array <- function(img, type=c('horizontal','vertical','both'))
{
    type <- match.arg(type)
    img <- imexpand(img)
    sz <- imgetdims(img)
    if ( type == 'horizontal' || type == 'both' )
        img <- img[,sz[1]:1,,drop=F]
    if ( type == 'vertical' || type == 'both' )
        img <- img[sz[2]:1,,,drop=F]
    return(imsqueeze(img))
}

#' transposes a 3D array
#' @param img source image to transpose
#' @return matrix of same size as original, transposed for each channel.
#' @family reshape
imtranspose.array <- function(img)
{
    n <- length(dim(img))
    return( aperm(img,c(2,(1:n)[-2])) )
}

#' Determines whether an image is thermal based on its extension.
#'
#' @param fName     the path to the file
#' @return boolean  whether or not the image could be a thermal file
#' @note This just tests that the extension of the file is '.bin', so a
#'  file for which the function returns ``TRUE`` might not actually be a
#'  thermal image file.
#' @export
#' @family thermal
isthermal <- function( fName ) {
    return(getExtension(fName)=='bin')
}

#' Loads a thermal image from a .bin file
#' @param filename the file name of the .bin file
#' @param size the width,height of the image (default 640x480 for the Thermoteknix Miricle)
#' @param max what maximum intensity to scale the image to.
#'                Put NA or 0 or NULL to not scale
#' @param return.what c("array","image"): whether to return an array of the image values,
#'                   or a Voir image (**if you have Voir**).
#' @return a LIST with components:
#'     \describe{
#'        \item{image}{The data/intensities of the thermal image (ie array or Voir image)}
#'        \item{detectorTemperature}{The temperature of the detector in the camera at time of image}.
#'     }
#' @export
#' @family thermal
imloadthermal <- function( filename, size=c(640,480),
                          max=imageUtilities.option('max'), 
                          return.what=c("array","image") )
{
    # open file.
    f <- file(filename,"rb")
    # 640x480 of 2-byte (16bit) ints.
    BIN <- readBin(f,n=prod(size),what='int',size=2)
    # 1 of 4-byte (32bit) float
    detectorTemp <- readBin(f,n=1,what=numeric(),size=4)
    close(f)


    imgAry <- matrix(BIN, nrow=size[1],ncol=size[2])

    # scale image
    if ( !missing(max) && !is.empty(max) && max>0 ) {
        # convert to 0....256 ?
        # a couple values of 0
        MIN <- min( imgAry[imgAry>0] )
        MAX <- base::max( imgAry )

        imgAry <- pmax((imgAry-MIN)/(MAX-MIN) * max,0)
    }
    
    # URGH this breaks the API hole between Voir/Non!
    outImg <- im2array(imgAry,transpose=TRUE)
    if ( tolower(return.what[1])=="image" ) {
        outImg <- array2im(outImg)
    }
    return( list( image=outImg, detectorTemperature=detectorTemp ) )
}

#' pads an *array* to 3 dimensions (i.e. sets third dimension to '1' if
#' it's single-channel).
#' @param x input array
#' @return input array exactly if \code{length(dim(x))==3}, otherwise the
#' input array with dimensions \code{ dim(x),1 } (i.e. 3D slice).
#' @family foobar
#' @keywords internal
#' @export
imexpand <- function(x)
{
    if ( length(dim(x))==2 )
        dim(x)<-c(dim(x),1)
    return(x)
}

#' drops singleton dimensions from an image (the same as \code{drop})
#' @param x input array
#' @return \code{drop(x)}
#' @seealso \code{\link[base]{drop}}
#' @family foobar
#' @keywords internal
#' @export
imsqueeze <- function(x)
{ 
    return(drop(x))
}

#' subsample an image to at most a certain size
#'
#' @param src input image to downsample
#' @param maxSize \code{c(width,height)} specifying the maximum size the
#' output image is permitted to be. NA for dimensions you don't care about.
#' @param method 'every' or 'mean'. See \code{\link{imsubsample}}.
#' @param strict whether to strictly enforce the image being at most
#' maxSize, or allow slightly bigger if that works better than the smaller
#' option.
#' @return input image subsampled such that it is at *most* maxSize.
#' @note The factor to downsample by must be an integer, so the integer
#' chosen is \code{ceiling(size/maxSize)} if \code{strict=TRUE}, and 
#' \code{round(size/maxSize)} otherwise. 
#'
#' Aspect ratio is preserved.
#' @seealso \code{\link{imsubsample}}.
#' @export
imsubsample2 <- function(src,maxSize,method=c('every','mean'),
                         strict=TRUE)
{
    if ( all(is.na(maxSize)) )
        stop("at least one of maxSize can't be empty")
    sz <- imgetdims(src)[1:2]
    if ( strict )
        FACT <- max(ceiling(sz/maxSize),na.rm=TRUE)
    else
        FACT <- max(round(sz/maxSize),na.rm=TRUE)

    if ( FACT>1 )
        return( imsubsample(src,FACT,FACT,method=method) )
    else
        return(src)
}

