#' loads an image from a file path.
#'
#' @param  imageOrPath the path to the image to load.
#' @param  max what to scale the output image to.
#' @param  greyscale whether the image is greyscale or not. 
#'         All it does is return \code{imsqueeze} on the first channel of the
#'         image.
#' @return an array, (ny,nx,numchanns).
#' @note supported formats: jpeg, tiff (require Voir), bin (ie thermal),
#'       png (requires library(png))
#'
#' @family io
#' @export
imload <- function( imageOrPath,
                   max=imageUtilities.option('max'),
                   greyscale=FALSE )
#  loads an image from a filepath. if an image is supplied then it just returns that.
{
    img <- NULL
    # see if the input is an img or path
    if ( is.image(imageOrPath) ) {
        img <- imageOrPath
    } else if ( is.character(imageOrPath) ) {
        # Get Image & error check.
        ext <- getExtension( imageOrPath )
        supportedExts <- c('png','jpeg','jpg','tiff','tif','bin')
        tryCatch({
            ext <- match.arg(ext, supportedExts)
        }, error = function(e) {stopf('imload: unsupported/unknown extension %s\n',ext) })

        if ( max == 1 )
            f <- identity
        else
            f <- function(x) x*max

        if ( any(grep( "jpe?g",ext )) ) {
            library(jpeg)
            img <- f(readJPEG(imageOrPath))
        } else if ( any(grep( "tiff?",ext )) ) {
            library(gridExtra)
            library(pixmap)
            img <- f(getChannels(read.tiff(imageOrPath)))
        } else if ( any(grep("bin",ext)) ) {
            img <- imloadthermal(imageOrPath,return.what='image',max=max)$image
        } else if ( ext == "png" ) {
            library(png)
            img <- f(readPNG(imageOrPath))
        }
    }
    if ( isTRUE(greyscale) )
        img <- imsqueeze(img[,,1])
    return(img)
}

#' saves an image (tiff, jpeg, png, bmp)
#' @param img     the image to save
#' @param name    the file name to save as
#' @param comment save this as the image's comment (if JPEG or TIFF)
#' @param qual    quality of the file (if JPEG, default 100)
#' @param max     max intensity of input image
#' @note bmp and tiff use the base \code{\link[grDevices]{png}} and
#' \code{\link[grDevices]{tiff}} functions in combination with
#' (internal to this package) \code{\link{imsave.grid}}, which needs
#' package 'grid'.
#' The png and jpeg packages are used to write those images.
#' @seealso \code{\link[png]{writePNG}}, \code{\link[jpeg]{writeJPEG}}
#' @family io
#' @export
imsave <- function( img, name, comment=NULL, qual=100,
                   max=imageUtilities.option('max') )
{
    ext <- getExtension( name )
    ext <- switch(ext, png='png',
                       jpeg=,jpg='jpeg',
                       tif=,tiff='tiff',
                       bmp='bmp',
                       stop("imsave: unsupported/unknown extension %s\n",
                            ext))

    if ( muffle.messages(require(grid)) ) {
        if ( ext == 'jpeg' )
            imsave.grid(img, name, FUN=ext, max=max, quality=qual)
        else
            imsave.grid(img, name, FUN=ext, max=max)
    } else if ( ext =='png' ) {
        library(png)
        writePNG(img/max,name)
    } else if ( ext == 'jpeg' ) {
        library(jpeg)
        writeJPEG(img/max,name)
    } else {
        stop("imsave: couldn't load required library grid")
    }
    return()
}




if ( !is.null(getProgram(c('exiftool','exiftool.exe'))) ) { 
#' sets an image's comment (JPEG and TIFF only), using 'exiftool'
#' @param name    file name of image you want to set the comment of
#' @param comment the comment to add.
#' @note this sets the 'ImageDescription' tiff tag of a tiff file, and the
#' 'comment' tag of a jpeg file.
#' 
#' This relies on a system call to executable 'exiftool'. If it can't be 
#' found, then the function will not work!
#' @export
imsetcommentsystem <- function( name, comment ) 
{
    EXIFTOOL <- getProgram(c('exiftool','exiftool.exe'))[0]
    ext <- getExtension(name)
    if ( any(grep("jpe?g",ext)) ) { 
                tag <- "Comment"
        } else if ( any(grep( "tiff?",ext )) ) { 
                    tag <- "ImageDescription"
            } else { 
                        stop("unrecognised extension")
                } 
        # escape double quotes, don't know if i need an extra layer of '\' for shell though?
        cmt <- gsub('([^\\\\]|^)"','\\1\\"',comment,perl=TRUE)
            
            res <- system(sprintf('%s -%s="%s" %s',EXIFTOOL,tag,cmt,name))
            return(res)
}
} else {
#' Does nothing (you don't have exiftool or exiftool.exe in your PATH; add
#' it and reinstall the package).
#' @param ... unused
#' @note This function uses exiftool to set the comment of an image. If you
#' see this help page, you don't have exiftool installed or it's not in
#' your $PATH. To use this function, install exiftool, add it to your path,
#' and reinstall this package.
#' @export
imsetcommentsystem <- function(...) { return(invisible(NULL)) }
}

